using UnityEngine;
using System;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.IO;

// LightsabersEditor v1.0
//
// Allows you to edit some parameters of Lightsaber particles (circle type)

namespace LightsabersEditorPlugin
{
    public class LightsabersEditor : MVRScript
    {
		public JSONStorableColor LightsaberMainColor;
		public JSONStorableFloat LightsaberType;
		public JSONStorableBool LightsaberEnableLightning;
		public JSONStorableBool LightsaberEnableTendrils;
		
		public Transform ObjectRoot;
		public Transform HiltsRoot;
		public Transform currentHilt;
		private string objectNameSearchString = "lightsaber";

		private bool seInitialized = false;
		
		// Security
		private int initLoops = 0;
		private int maxInitLoops = 3600;
		
		public override void Init()
        {			 
            try
            {
				if (containingAtom.type != "CustomUnityAsset")
                {
                    SuperController.LogError("Please add LightsabersEditor on an Custom Unity Asset with a compatible particle system.");
                    return;
                }

				HSVColor hsvc = HSVColorPicker.RGBToHSV(0.3f, 1f, 0.34f);
				LightsaberMainColor = new JSONStorableColor("Lightsaber color", hsvc, MainColorCallback);
				RegisterColor(LightsaberMainColor);
				CreateColorPicker(LightsaberMainColor);
							
				LightsaberType = new JSONStorableFloat("Lightsaber type", 1f, (val) => { LightsaberType.valNoCallback = Mathf.Round(val); TypeCallback( LightsaberType.val ); }, 1f, 6f);
				RegisterFloat(LightsaberType);
				CreateSlider(LightsaberType, false);
				
				LightsaberEnableLightning = new JSONStorableBool("Enable lightning", false, EnableVFXCallback);
				RegisterBool(LightsaberEnableLightning);
				CreateToggle(LightsaberEnableLightning, false);
				
				LightsaberEnableTendrils = new JSONStorableBool("Enable tendrils", false, EnableVFXCallback);
				RegisterBool(LightsaberEnableTendrils);
				CreateToggle(LightsaberEnableTendrils, false);		
							
				JSONStorableString helpText = new JSONStorableString("Help",
					"<color=#000><size=35><b>Help</b></size></color>\n\n" + 
					"<color=#333>" +
					"<b>LightsabersEditor</b> is a tiny plugin that allows you to edit some parameters of the Lightsaber inside this package\n\n" +
					"Just test the parameters on the left, and the lightsaber will update in realtime.\n\n" +
					"<b>IMPORTANT :</b> If you switch lightsabers (at the moment, there is only one, but who knows...) in the Asset tab of the CUA, don't forget to reload the script to enable customization.\n\n" +
					"</color>"
				);
				UIDynamic helpTextfield = CreateTextField(helpText, true);
				helpTextfield.height = 550.0f;
				

			}
            catch(Exception e)
            {
                SuperController.LogError("LightsabersEditor - Exception caught: " + e);
            }
		}
		
		void Start()
        {

        }
		
		void Update()
        {
			// If we haven't made over 1min of init tests, we aren't initialized and the particle system (CUA) is still compatible, try to initialize
			if( initLoops < maxInitLoops && seInitialized == false ) {
				initLoops++;
				InitObjects();
			}
			
			if( initLoops == maxInitLoops ) {
				logError("LightsabersEditor : This CUA particle could not be initialized properly after 3600 tries. Please select a compatible particle and reload the script.");
				initLoops++; // To avoid spamming the log
			}
		}
		
		private bool InitObjects() {
			if( seInitialized == true ) return true;
			
			ObjectRoot = findObjectRoot( containingAtom.reParentObject );
			if( ObjectRoot == null ) {
				return false;
			}
			
			HiltsRoot = getObjectRoot( ObjectRoot, "hilts" );
			if( HiltsRoot != null ) {
				seInitialized = true;
				
				TypeCallback( LightsaberType.val );
				
				MainColorCallback( LightsaberMainColor );
				
								
				logDebug("LightsabersEditor : Lightsaber editor initialized properly");
							
				return true;
			} else {
				logError("LightsabersEditor : your CUA is not a lightsaber.");
				initLoops = maxInitLoops; // We don't have a particle system, we consider the initialization failed
				return false;
			}
		}
		
		// Finds the object containing a portion of a name
		private Transform findObjectRoot( Transform parent ) {
			foreach( Transform child in parent )
			{
				if( child.name.Contains(objectNameSearchString) ) {
					return child;
				} else {
					Transform childSearch = findObjectRoot(child);
					if (childSearch != null) return childSearch;
				}
			}
			
			return null;
		}
		
		// Finds the object with an exact name
		private Transform getObjectRoot( Transform parent, string objectName ) {
			foreach( Transform child in parent )
			{
				if( child.name == objectName ) {
					return child;
				} else {
					Transform childSearch = getObjectRoot(child, objectName);
					if (childSearch != null) return childSearch;
				}
			}
			
			return null;
		}
		
		protected void MainColorCallback(JSONStorableColor color) {
			if( currentHilt ) {
				initLightsaber();
			}
		}
		
		protected void TypeCallback(float type) {
			if( HiltsRoot ) {
				disableAllHilts();
				currentHilt = enableHilt(type);
				initLightsaber();
			}
		}
		
		protected void EnableVFXCallback(bool enabled) {
			if( currentHilt ) {
				initVisualEffects();
			}
		}
		
		protected void disableAllHilts() {
			foreach( Transform child in HiltsRoot )
			{
				child.gameObject.SetActive(false);
			}
		}
		
		protected Transform enableHilt(float hiltValue) {
			foreach( Transform child in HiltsRoot )
			{
				if( child.name == "hilt_0" + hiltValue ) {
					child.gameObject.SetActive(true);
					
					return child;
				}
			}
			
			return null;
		}
		
		protected void initLightsaber() {
			initBladeColor();
			initBladeGlow();
			initVisualEffects();
		}
		
		protected void initBladeColor() {
			Transform blade = getObjectRoot( currentHilt, "blade" );
			if( blade ) {
				Renderer bladeRenderer = blade.GetComponent<Renderer>();
				bladeRenderer.material.SetColor("_BladeColor",new Color(LightsaberMainColor.colorPicker.currentColor.r, LightsaberMainColor.colorPicker.currentColor.g, LightsaberMainColor.colorPicker.currentColor.b, 1.0f) );
			}
		}
		
		protected void initBladeGlow() {
			Transform bladeGlow = getObjectRoot( currentHilt, "blade_glow" );
			if( bladeGlow ) {
				ParticleSystem prtComponent = bladeGlow.GetComponent<ParticleSystem>();
				if( prtComponent ) {
					Color newColor = new Color(LightsaberMainColor.colorPicker.currentColor.r, LightsaberMainColor.colorPicker.currentColor.g, LightsaberMainColor.colorPicker.currentColor.b, 1.0f);
					var prtMain = prtComponent.main;
					prtMain.startColor = newColor;
					prtComponent.Clear();
					prtComponent.Play();
				}
			}
		}
		
		protected void initVisualEffects() {
			Transform bladeLightning = getObjectRoot( currentHilt, "blade_lightning" );
			if( bladeLightning ) {
				bladeLightning.gameObject.SetActive( LightsaberEnableLightning.val );
				ParticleSystem prtComponentLightning = bladeLightning.GetComponent<ParticleSystem>();
				if( prtComponentLightning ) {
					Color newColor = new Color(LightsaberMainColor.colorPicker.currentColor.r, LightsaberMainColor.colorPicker.currentColor.g, LightsaberMainColor.colorPicker.currentColor.b, 1.0f);
					var prtMain = prtComponentLightning.main;
					prtMain.startColor = newColor;
					prtComponentLightning.Clear();
					prtComponentLightning.Play();
				}
			}
			
			Transform bladeTendrils = getObjectRoot( currentHilt, "blade_tendrils" );
			if( bladeTendrils ) {
				bladeTendrils.gameObject.SetActive( LightsaberEnableTendrils.val );
				ParticleSystem prtComponentTendrils = bladeTendrils.GetComponent<ParticleSystem>();
				if( prtComponentTendrils ) {
					Color newColor = new Color(LightsaberMainColor.colorPicker.currentColor.r, LightsaberMainColor.colorPicker.currentColor.g, LightsaberMainColor.colorPicker.currentColor.b, 1.0f);
					var prtMain = prtComponentTendrils.main;
					prtMain.startColor = newColor;
					prtComponentTendrils.Clear();
					prtComponentTendrils.Play();
				}
			}
		}
		
		// **************************
		// Local Tools
		// **************************
		private void logDebug( string debugText ) {
			SuperController.LogMessage( debugText );
		}
		
		private void logError( string debugText ) {
			SuperController.LogError( debugText );
		}
	}
}